// importing necessary modules I used for this graph.
import { getData } from './getData';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format,
} from 'd3';

// declaring constants by reading svg tag from index.html
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');
const xAxisTickFormat = (number) => format('.4~g')(number);

// Start rendering pie chart to html
const render = (data, total) => {
  //
  const pieData = pie().value((d) => d.Amount)(data);

  // Colors for in Pie Chart
  const colors = scaleOrdinal()
    .domain(data)
    .range([
      '#1E88E5',
      '#2E7D32',
      '#F57C00',
      '#BA68C8',
      '#D32F2F',
      '#90CAF9',
      '#81C784',
      '#FFEB3B',
      '#E1BEE7',
      '#F48FB1',
      '#9400D3',
    ]);

  const segments = arc()
    .innerRadius(0)
    .outerRadius(170)
    .padAngle(0)
    .padRadius(0);
  // Rendering section of pie chart
  const sections = svg
    .append('g')
    .attr('transform', `translate(250,250)`)
    .selectAll('path')
    .data(pieData);

  // Rendering color to the section to pie chart
  sections
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d) => colors(d.data.Amount));

  // Redering legends for pie chart
  const legends = svg
    .append('g')
    .attr('transform', 'translate(500,100)')
    .selectAll('.legends')
    .data(pieData);
  const legend = legends
    .enter()
    .append('g')
    .classed('.legends', true)
    .attr('transform', (d, i) => {
      return `translate(0,${(i + 1) * 25})`;
    });

  //redering legend colors to pie chart
  legend
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d) => colors(d.data.Amount));

  //Redering legends sales rep
  legend
    .append('text')
    .attr('x', 25)
    .attr('y', 15)
    .attr('class', 'legend_text')
    .text((d) => d.data.SalesRep);

  //Redering legends sales value percentage text
  legend
    .append('text')
    .attr('x', 170)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text(
      (d) =>
        xAxisTickFormat(
          ((d.data.Amount / total) * 100).toFixed(1),
        ) + '%',
    );
};

// load data from github link by using helper class getData
const run = async () => {
  let data = await getData();
  let total = 0;
  data.forEach((d) => {
    d.Amount = +d.Amount;
    total += d.Amount;
  });
  render(data, total);
};

//Initial Function call to render pie chart
run();
